/*  
 * Copyright (c) 2005 Blueprint Software Works
 *                    http://www.blueprintsoftwareworks.com
 * Copyright (c) 2005 Vitali E. Pelenyov
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files
 * (the "Software"), to deal in the Software without restriction,
 * including without limitation the rights to use, copy, modify, merge,
 * publish, distribute, sublicense, and/or sell copies of the Software,
 * and to permit persons to whom the Software is furnished to do so,
 * subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
 * IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
 * CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 * TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH
 * THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE. 
 */

#ifndef __RESOURCE__H
#define __RESOURCE__H

#include "ndp.hpp"

#include <config.h>

#include <dav.h>

#include <ne_request.h>
#include <ne_auth.h>
#include <ne_basic.h>
#include <ne_string.h>
#include <ne_props.h>
#include <ne_uri.h>
#include <ne_socket.h>
#include <ne_dates.h>

int ResourceInit (void);
int ResourceShutdown (void);

class Session
{
    private:
        ne_session *m_sess;
    public:
        Session (ne_session *sess) : m_sess (sess) {};
        ~Session ()
        {
            if (m_sess) ne_session_destroy (m_sess);
        };
        
        ne_session *s (void) { return m_sess; };
};

class Resource : public NdResource
{
    private:
        ne_uri uri;
        int fURIPathIsRoot;
    
    public:
        Resource (NDPROPERTYHANDLE p);
        virtual ~Resource (void);
        
        virtual int Initialize (void);
        virtual int EqualTo (NdResource *pres);
        virtual int Available (void);
        virtual int QueryInfoString (char **ppinfo);
        virtual void ReleaseInfoString (char *pinfo);
        
        Session *CreateSession (void);
        void DeleteSession (Session *sess);

        virtual const char *QueryFullPath (const char *path);
        virtual void ReleaseFullPath (const char *fullpath);
        
        const char *QueryURI (const char *fullpath);
        void ReleaseURI (const char *res_uri);
};

class Connection : public NdConnection
{
    private:
        Session *sess;
        char *cookie;
        
    public:
        Connection (Resource *pres);
        virtual ~Connection (void);
        
        virtual int Initialize (void);
        
        virtual int Refresh (const char *path, int tree);
    
        virtual int QueryPathInfo (NdPathInfo *ppathinfo, const char *path);
        
        virtual int SetPathInfo (const char *path, NdPathInfo *ppathinfo);
        virtual int SetCurrentDir (const char *path);
        
        virtual int Copy (const char *dst, const char *src, ULONG ulOption);
        virtual int CopyTo (NdResource *resdst, const char *dst, const char *src, ULONG ulOption);
        
        virtual int DeleteFile (const char *path);
        
        virtual int CreateDir (const char *path);
        
        virtual int List (NdPathInfo **pppathinfo, const char *path);
        
        virtual int Move (const char *dst, const char *src);
        virtual int MoveTo (NdResource *resdst, const char *dst, const char *src);

        virtual int ChangeCase (const char *dst, const char *src, const char *newname, ULONG namelen);
        virtual int Rename (const char *dst, const char *src, const char *newname, ULONG namelen);
    
        virtual int Open (NdFile *pfile);
        virtual int OpenReplace (NdFile *pfile);
        virtual int OpenCreate (NdFile *pfile);
        
        virtual int DeleteDir (const char *path);
    
        virtual int Read (const char *szFile, LONGLONG llOffset, void *pbuffer, ULONG ulRead, ULONG *pulActual);
        virtual int Write (const char *szFile, LONGLONG llOffset, void *pbuffer, ULONG ulWrite, ULONG *pulActual);
        
        void GetAuthInfo (char *username, char *password);
        void AddCookie (ne_request *req, ne_buffer *header);
        void SetCookie (ne_request *req, const ne_status *status);
     
        Resource *GetResource (void) { return (Resource *)NdConnection::GetResource (); };
        
};

#endif /* __RESOURCE__H */
